﻿Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel
Imports Microsoft.IdentityModel.Claims
Imports Microsoft.IdentityModel.Configuration
Imports Microsoft.IdentityModel.Protocols.WSTrust
Imports Microsoft.IdentityModel.SecurityTokenService

Namespace TestWorkflowService
	Friend Class CustomSecurityTokenService
		Inherits SecurityTokenService
		Public Sub New(ByVal config As SecurityTokenServiceConfiguration)
			MyBase.New(config)
		End Sub

        Protected Overrides Function GetScope(ByVal principal As IClaimsPrincipal,
                                              ByVal request As RequestSecurityToken) As Scope
            Dim appliesTo = request.AppliesTo.Uri.AbsoluteUri
            Dim scope As New Scope(appliesTo, SecurityTokenServiceConfiguration.SigningCredentials)

            scope.ReplyToAddress = request.ReplyTo
            If String.IsNullOrEmpty(scope.ReplyToAddress) Then scope.ReplyToAddress = request.AppliesTo.Uri.AbsoluteUri

            scope.EncryptingCredentials = New X509EncryptingCredentials(GetRPCert(request.AppliesTo))
            Return scope
        End Function

		Private Shared Function GetRPCert(ByVal appliesTo As EndpointAddress) As X509Certificate2
			Select Case appliesTo.Uri.AbsoluteUri
				Case Else
					Return New X509Certificate2("localhost.cer")
			End Select
		End Function

		Private Sub ValidateAppliesTo(ByVal appliesTo As EndpointAddress)
			If appliesTo Is Nothing Then Throw New InvalidRequestException("The appliesTo is null.")
		End Sub

        Public Overrides Function Issue(ByVal principal As IClaimsPrincipal,
                                        ByVal request As RequestSecurityToken) As RequestSecurityTokenResponse
            Console.WriteLine("Issuing token...")
            Dim token = MyBase.Issue(principal, request)
            Return token
        End Function

        Protected Overrides Function GetOutputClaimsIdentity(ByVal principal As IClaimsPrincipal,
                                                             ByVal request As RequestSecurityToken,
                                                             ByVal scope As Scope) As IClaimsIdentity
            ValidateAppliesTo(request.AppliesTo)

            ' Create new identity and copy content of the caller's identity into it (including the existing delegate chain)
            Dim callerIdentity As IClaimsIdentity = CType(principal.Identity, IClaimsIdentity)
            Dim outputIdentity As IClaimsIdentity = callerIdentity.Copy()

            ' If there is an ActAs token in the RST, return a copy of it as the top-most identity
            ' and put the caller's identity into the Actor property of this identity.
            If request.ActAs IsNot Nothing Then
                Dim actAsSubject As IClaimsIdentity = request.ActAs.GetSubject()(0)
                Dim actAsIdentity As IClaimsIdentity = actAsSubject.Copy()

                ' Find the last actor in the actAs identity
                Dim lastActor As IClaimsIdentity = actAsIdentity
                Do While lastActor.Actor IsNot Nothing
                    lastActor = lastActor.Actor
                Loop

                ' Set the caller's identity as the last actor in the delegation chain
                lastActor.Actor = outputIdentity

                ' Return the actAsIdentity instead of the caller's identity in this case
                outputIdentity = actAsIdentity
                Console.WriteLine("Creating ActAs claims...")
            End If
            If outputIdentity.Claims.Count = 0 Then

                outputIdentity.Claims.AddRange(
                    New Claim() {
                        New Claim(IdentityModel.Claims.ClaimTypes.Name, "Bob Creamer"),
                        New Claim(IdentityModel.Claims.ClaimTypes.Email, "bobc@fabrikam.com"),
                        New Claim(IdentityModel.Claims.ClaimTypes.Upn, "bobc@fabrikam.com")})
            End If
            Return outputIdentity
        End Function
	End Class
End Namespace