﻿Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel
Imports Microsoft.IdentityModel.Claims
Imports Microsoft.IdentityModel.Configuration
Imports Microsoft.IdentityModel.Protocols.WSTrust
Imports Microsoft.IdentityModel.SecurityTokenService


Namespace BasicSTS

	Friend Class CustomSecurityTokenService
		Inherits SecurityTokenService
		Private _addressExpected As String = "http://some"
		Public Sub New(ByVal config As SecurityTokenServiceConfiguration)
			MyBase.New(config)
		End Sub

        Protected Overrides Function GetScope(ByVal principal As IClaimsPrincipal,
                                              ByVal request As RequestSecurityToken) As Scope
            Dim appliesTo = request.AppliesTo.Uri.AbsoluteUri
            Dim scope As New Scope(appliesTo, SecurityTokenServiceConfiguration.SigningCredentials)

            scope.ReplyToAddress = request.ReplyTo
            If String.IsNullOrEmpty(scope.ReplyToAddress) Then
                scope.ReplyToAddress = request.AppliesTo.Uri.AbsoluteUri
            End If

            'Public key from this cert will be used to encrypt the token.
            'only public key cert. use to secure communication.
            scope.EncryptingCredentials = New X509EncryptingCredentials(GetRPCert(request.AppliesTo))

            Return scope
        End Function

		Private Shared Function GetRPCert(ByVal appliesTo As EndpointAddress) As X509Certificate2
			If appliesTo.Uri.AbsoluteUri.EndsWith("WFFacade") Then
				Return New X509Certificate2("WFServiceFacade.cer")
			End If

			Return New X509Certificate2("SampleService.cer")
		End Function

		Private Sub ValidateAppliesTo(ByVal appliesTo As EndpointAddress)
			If appliesTo Is Nothing Then
				Throw New InvalidRequestException("The appliesTo is null.")
			End If

			If Not appliesTo.Uri.Equals(New Uri(_addressExpected)) Then
                Throw New InvalidRequestException(
                    String.Format("The relying party address is not valid. Expected value is {0}, the actual value is {1}.",
                                  _addressExpected,
                                  appliesTo.Uri.AbsoluteUri))
			End If
		End Sub

        Public Overrides Function Issue(ByVal principal As IClaimsPrincipal,
                                        ByVal request As RequestSecurityToken) As RequestSecurityTokenResponse
            Console.WriteLine("Issuing token...")
            Dim token = MyBase.Issue(principal, request)
            Return token
        End Function


        Protected Overrides Function GetOutputClaimsIdentity(ByVal principal As IClaimsPrincipal,
                                                             ByVal request As RequestSecurityToken,
                                                             ByVal scope As Scope) As IClaimsIdentity
            'ValidateAppliesTo(request.AppliesTo)

            ' Create new identity and copy content of the caller's identity into it (including the existing delegate chain)
            Dim callerIdentity = CType(principal.Identity, IClaimsIdentity)
            Dim outputIdentity As IClaimsIdentity = callerIdentity.Copy()


            ' If there is an ActAs token in the RST, return a copy of it as the top-most identity
            ' and put the caller's identity into the Actor property of this identity.
            If request.ActAs IsNot Nothing Then
                Dim actAsSubject As IClaimsIdentity = request.ActAs.GetSubject()(0)
                Dim actAsIdentity As IClaimsIdentity = actAsSubject.Copy()

                ' Find the last actor in the actAs identity
                Dim lastActor = actAsIdentity
                Do While lastActor.Actor IsNot Nothing
                    lastActor = lastActor.Actor
                Loop

                ' Set the caller's identity as the last actor in the delegation chain
                lastActor.Actor = outputIdentity

                ' Return the actAsIdentity instead of the caller's identity in this case
                outputIdentity = actAsIdentity
            End If
            If outputIdentity.Claims.Count = 0 Then 'add hardcoded claims...
                outputIdentity.Claims.AddRange(
                    New Claim() {New Claim(IdentityModel.Claims.ClaimTypes.Name, "Zulfiqar"),
                                 New Claim(IdentityModel.Claims.ClaimTypes.Email, "me@zamd.net"),
                                 New Claim(IdentityModel.Claims.ClaimTypes.Upn, "zamd@zamd.net")})
            End If
            Return outputIdentity
        End Function
	End Class
End Namespace