﻿Imports System.IdentityModel.Selectors
Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Security
Imports Extensions.Utilities
Imports Microsoft.IdentityModel.Configuration
Imports Microsoft.IdentityModel.Protocols.WSTrust
Imports Microsoft.IdentityModel.SecurityTokenService
Imports Microsoft.IdentityModel.Tokens

Namespace BasicSTS


	Friend Class Program
		Shared Sub Main(ByVal args() As String)
			Dim stsSignCert As New X509Certificate2("BasicSts.pfx","a")

            Dim stsConfig As SecurityTokenServiceConfiguration = New SecurityTokenServiceConfiguration With
                            {.ServiceCertificate = stsSignCert,
                             .SigningCredentials = New X509SigningCredentials(stsSignCert),
                             .SecurityTokenService = GetType(CustomSecurityTokenService),
                             .TokenIssuerName = "urn:zamdBasicSts"}

            stsConfig.SecurityTokenHandlers.AddOrReplace(
                New CustomUserNameSecurityTokenHandler With {
                    .UserNamePasswordValidator = Sub(uid As String, pwd As String)
                                                     Console.WriteLine("{0} authenticated.", uid)
                                                 End Sub})

            Dim actAsConfig As ServiceConfiguration = New ServiceConfiguration With {
                .AudienceRestriction = New AudienceRestriction(AudienceUriMode.Never),
                .CertificateValidationMode = X509CertificateValidationMode.None,
                .IssuerNameRegistry = New MyTestRegistry()}
            actAsConfig.Initialize()

            stsConfig.SecurityTokenHandlerCollectionManager(SecurityTokenHandlerCollectionManager.Usage.ActAs) =
                actAsConfig.SecurityTokenHandlers

            Dim stsHost = New WSTrustServiceHost(stsConfig)
            stsHost.Open()


            Console.WriteLine(stsHost.WriteEndpoints())
            Console.ReadLine()

            stsHost.Close()
        End Sub
    End Class

    Public Class MyTestRegistry
        Inherits IssuerNameRegistry
        Public Overloads Overrides Function GetIssuerName(ByVal securityToken As System.IdentityModel.Tokens.SecurityToken) As String
            Return "TestIssuer"
        End Function
    End Class
End Namespace
