﻿Imports System.Text
Imports Microsoft.IdentityModel.Tokens
Imports Microsoft.IdentityModel.Claims
Imports System.IdentityModel.Tokens
Imports System.Xml

Namespace Extensions.Utilities
	'Enables callback based UserName/Password validation - Similar to WCF UserNamePasswordValidator.
	Public Class CustomUserNameSecurityTokenHandler
        Inherits UserNameSecurityTokenHandler

        Public Property UserNamePasswordValidator() As Action(Of String, String)
        Public Sub New()
            RetainPassword = True
        End Sub

        Public Overrides ReadOnly Property CanValidateToken() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides Function ValidateToken(ByVal token As SecurityToken) As ClaimsIdentityCollection
            Dim userNameToken = TryCast(token, UserNameSecurityToken)
            If userNameToken Is Nothing Then
                Throw New ArgumentException("Invalid token argument.")
            End If

            If Me.UserNamePasswordValidator Is Nothing Then
                Diagnostics.Trace.TraceWarning("Custom validator is missing. UserNameSecurityToken cannot be validated.")
                Throw New InvalidOperationException("Custom validator is missing. " & "Please specify your custom validator using the UserNamePasswordValidator property of CustomUserNameSecurityTokenHandler")
            End If

            Me.UserNamePasswordValidator()(userNameToken.UserName, userNameToken.Password)

            Dim identity As IClaimsIdentity = New ClaimsIdentity(
                                              New Claim() {New Claim(ClaimTypes.Name, userNameToken.UserName)},
                                              AuthenticationTypes.Password)
            Dim Claims As Microsoft.IdentityModel.Claims.ClaimCollection =
                {New Claim(ClaimTypes.AuthenticationInstant,
                           XmlConvert.ToString(Date.UtcNow, "yyyy-MM-ddTHH:mm:ss.fffZ"),
                           ClaimValueTypes.Datetime),
                 New Claim(ClaimTypes.AuthenticationMethod, AuthenticationMethods.Password)}

            If Configuration.SaveBootstrapTokens Then
                If RetainPassword Then
                    identity.BootstrapToken = userNameToken
                Else
                    identity.BootstrapToken = New UserNameSecurityToken(userNameToken.UserName, Nothing)
                End If
            End If
            Return New ClaimsIdentityCollection(New IClaimsIdentity() {identity})
        End Function

        Public Overrides Function Clone() As Microsoft.IdentityModel.Tokens.SecurityTokenHandler
            Return Nothing
        End Function
    End Class
End Namespace
