﻿Imports System.Activities.Tracking
Imports System.ServiceModel
Imports System.ServiceModel.Activities
Imports System.ServiceModel.Description
Imports System.Text
Imports System.Xml

Namespace Extensions.Utilities
	Public Module ServiceHostBaseExtensions
		Private Function CreateOrRemoveMetadataBehavior(ByVal host As ServiceHostBase) As ServiceMetadataBehavior
			Dim metadataBv = host.Description.Behaviors.Remove(Of ServiceMetadataBehavior)()
			If metadataBv Is Nothing Then
				metadataBv = New ServiceMetadataBehavior()
			End If
			Return metadataBv
		End Function

        <System.Runtime.CompilerServices.Extension()>
        Public Sub ConfigureMetadataViaHttpGet(ByVal source As ServiceHostBase)
            ConfigureMetadataViaHttpGet(source, "")
        End Sub

        <System.Runtime.CompilerServices.Extension()>
        Public Sub ConfigureMetadataViaHttpGet(ByVal source As ServiceHostBase, ByVal url As String)
            Dim metadataBv = CreateOrRemoveMetadataBehavior(source)

            metadataBv.HttpGetEnabled = True
            If Not String.IsNullOrEmpty(url) Then
                metadataBv.HttpGetUrl = New Uri(url)
            End If

            source.Description.Behaviors.Add(metadataBv)

        End Sub

        <System.Runtime.CompilerServices.Extension()>
        Public Sub ConfigureMetadataViaMex(ByVal source As ServiceHostBase, ByVal address As String)
            Dim metadataBv = CreateOrRemoveMetadataBehavior(source)
            source.Description.Behaviors.Add(metadataBv)

            source.AddServiceEndpoint("IMetadataExchange", MetadataExchangeBindings.CreateMexHttpBinding(), address)
        End Sub

        <System.Runtime.CompilerServices.Extension()>
        Public Function WriteEndpoints(ByVal source As ServiceHostBase) As String
            Dim sb = New StringBuilder()
            Dim counter = 0

            sb.Append(vbLf & "Host information:")
            sb.Append(vbLf & "-----------------" & vbLf & vbLf)
            For Each ep In source.Description.Endpoints
                counter += 1
                sb.Append(String.Format("{0}>> ListenUri: {1}" & vbLf, counter, ep.ListenUri.ToString()))
                sb.Append(String.Format("Name: {0}, Endpoint Address: {1} Binding: {2}" & vbCrLf,
                                        ep.Name,
                                        ep.Address.ToString(),
                                        ep.Binding.ToString()))
            Next ep
            Dim meta = source.Description.Behaviors.Find(Of ServiceMetadataBehavior)()
            If meta IsNot Nothing AndAlso meta.HttpGetEnabled Then
                Dim metaUrl = If(meta.HttpGetUrl IsNot Nothing, meta.HttpGetUrl, source.BaseAddresses(0))
                sb.Append("*>> Http metadata URL: " & metaUrl.ToString() & vbLf)
            End If

            Return sb.ToString()

        End Function

        <System.Runtime.CompilerServices.Extension()>
        Public Sub WriteEndpointsBinding(ByVal source As ServiceHostBase)
            Dim orignal = Console.ForegroundColor
            Console.ForegroundColor = ConsoleColor.Cyan
            Dim count = 1
            For Each ep In source.Description.Endpoints
                Dim epr = "Endpoint " & count.ToString() & " : " & ep.ListenUri.ToString()
                Console.WriteLine(epr)
                For i = 0 To epr.Length - 1
                    Console.Write("=")
                Next i
                Console.WriteLine()
                For Each be In ep.Binding.CreateBindingElements()
                    Console.WriteLine(be.ToString())
                Next be
                Console.WriteLine()
                count += 1
            Next ep
            Console.ForegroundColor = orignal
            Console.WriteLine()
        End Sub

        <System.Runtime.CompilerServices.Extension()>
        Public Sub WriteServiceBahviors(ByVal source As ServiceHostBase)
            Dim orignal = Console.ForegroundColor
            Console.ForegroundColor = ConsoleColor.Green

            For Each bv In source.Description.Behaviors
                Console.WriteLine(bv.ToString())
            Next bv

            Console.ForegroundColor = orignal

        End Sub

        <System.Runtime.CompilerServices.Extension()>
        Public Sub LogUnknownMessages(ByVal source As ServiceHostBase)
            AddHandler source.UnknownMessageReceived,
                Sub(sender As Object, e As UnknownMessageReceivedEventArgs)
                    Dim orignal = Console.ForegroundColor
                    Console.ForegroundColor = ConsoleColor.Red

                    Console.WriteLine("Unknown message.")

                    Dim writer = XmlDictionaryWriter.Create(Console.OpenStandardOutput(),
                                                            New XmlWriterSettings With {.Indent = True})
                    e.Message.WriteMessage(writer)
                    writer.Flush()
                    writer.Close()
                    Console.WriteLine()

                    Console.ForegroundColor = orignal
                End Sub
        End Sub

        <System.Runtime.CompilerServices.Extension()>
        Public Sub EnableDebugTracking(ByVal source As WorkflowServiceHost)
            source.WorkflowExtensions.Add(New DebuggingTrackingParticipant())
        End Sub
	End Module

	Public Class DebuggingTrackingParticipant
		Inherits TrackingParticipant
        Public Sub New()
            Dim tmpWorkflowInstanceQuery As New WorkflowInstanceQuery

            tmpWorkflowInstanceQuery.States.Add("*")
            MyBase.TrackingProfile = New TrackingProfile()
            MyBase.TrackingProfile.Queries.Add(tmpWorkflowInstanceQuery)
            ' new ActivityScheduledQuery{ ActivityName="*"}
        End Sub

		Protected Overrides Sub Track(ByVal record As TrackingRecord, ByVal timeout As TimeSpan)
			If TypeOf record Is WorkflowInstanceUnhandledExceptionRecord Then

				Dim unRec = TryCast(record, WorkflowInstanceUnhandledExceptionRecord)
				Console.ForegroundColor = ConsoleColor.Red
				Console.WriteLine("Unhandled exception: " & unRec.UnhandledException.ToString())
				Console.ResetColor()
			ElseIf TypeOf record Is WorkflowInstanceAbortedRecord Then
				Console.ForegroundColor = ConsoleColor.Red
                Console.WriteLine("[*TR*] Workflow aborted. Reason = {0}",
                                  (CType(record, WorkflowInstanceAbortedRecord)).Reason)
				Console.ResetColor()
			ElseIf TypeOf record Is WorkflowInstanceRecord Then
				Dim instRecord = TryCast(record, WorkflowInstanceRecord)
				Console.WriteLine("[*TR*] Instance {0} state changed to :{1}",instRecord.InstanceId, instRecord.State)
			Else
				Console.WriteLine(record.GetType())
			End If
		End Sub
	End Class
End Namespace
