﻿Imports System.Text
Imports System.ServiceModel
Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Security
Imports System.ServiceModel.Description
Imports System.Activities
Imports System.ServiceModel.Activities

Namespace Extensions.Utilities

	Public Class Transform
		Public Property Name() As String
		Private action As Action(Of ServiceHostBase)
		Public Sub New(ByVal action As Action(Of ServiceHostBase))
			Me.action = action
        End Sub

		Public Sub New(ByVal name As String, ByVal action As Action(Of ServiceHostBase))
			Me.New(action)
			Me.Name = name
        End Sub

		Public Sub Apply(ByVal host As ServiceHostBase)
			action(host)
        End Sub

		Public Overrides Function ToString() As String
			Return Name
		End Function
	End Class

	Public Class ServiceHostManager
        Public Shared ReadOnly DebugTransform As Transform = New Transform("SetIncludeExceptionDetailInFaults",
      Sub(host)
          Dim debug = host.Description.Behaviors.Find(Of ServiceDebugBehavior)()
          If debug Is Nothing Then
              debug = New ServiceDebugBehavior()
              host.Description.Behaviors.Add(debug)
          End If
          debug.IncludeExceptionDetailInFaults = True
      End Sub)

        Public Const Prolog = "prologTransform"
        Public Const Epilog = "epilogTransform"

        Private freePort As Integer
        Private hostTransformMap As IDictionary(Of ServiceHostBase, List(Of Transform))

        Public Sub ServiceHostManager()
            Me.ServiceHostManager(8081)
        End Sub

        Public Sub ServiceHostManager(ByVal startPort As Integer)
            hostTransformMap = New Dictionary(Of ServiceHostBase, List(Of Transform))()
            Me.freePort = startPort
        End Sub

        Public Function RegisterActivity(ByVal root As Activity, ByVal httpBaseAddress As Uri) As ServiceHostBase
            If httpBaseAddress Is Nothing Then
                httpBaseAddress = New Uri(String.Format("http://localhost:{0}", freePort))
                freePort += 1
            End If
            Dim wfsh = New WorkflowServiceHost(root, httpBaseAddress)

            wfsh.ConfigureMetadataViaHttpGet()
            wfsh.EnableDebugTracking()
            hostTransformMap(wfsh) = New List(Of Transform)()

            ' default prolog & epilog for diagnostics...
            hostTransformMap(wfsh).Add(New Transform(Prolog,
                                                     Sub(host)
                                                         Console.WriteLine("'{0}' Host.", wfsh.Activity.DisplayName)
                                                         Console.WriteLine("<========================")
                                                     End Sub))

            hostTransformMap(wfsh).Add(New Transform(Epilog,
                                                     Sub(host)
                                                         Console.WriteLine(host.WriteEndpoints())
                                                         Console.WriteLine("========================/>")
                                                     End Sub))

            hostTransformMap(wfsh).Add(DebugTransform)

            Return wfsh
        End Function

        Public Function RegisterActivity(ByVal activity As Activity) As ServiceHostBase
            Return RegisterActivity(activity, Nothing)
        End Function

        Public Sub AddTransform(ByVal host As ServiceHostBase, ByVal transform As Transform)
            If hostTransformMap(host) Is Nothing Then
                hostTransformMap(host) = New List(Of Transform)()
            End If
            hostTransformMap(host).Add(transform)
        End Sub

        Public Sub AddTransform(ByVal host As ServiceHostBase, ByVal transform As Action(Of ServiceHostBase))
            AddTransform(host, New Transform(transform))
        End Sub

        Public Function RegisterService(ByVal serviceType As Type, ByVal httpBaseAddress As Uri) As ServiceHostBase
            If httpBaseAddress Is Nothing Then
                httpBaseAddress = New Uri(String.Format("http://localhost:{0}", freePort))
                freePort += 1
            End If

            Dim sh = New ServiceHost(serviceType, httpBaseAddress)

            hostTransformMap(sh) = New List(Of Transform)()
            ' default prolog & epilog transforms for diagnostics...
            hostTransformMap(sh).Add(New Transform(Prolog,
                                                   Sub(host)
                                                       Console.WriteLine("'{0}' Host.", serviceType.FullName)
                                                       Console.WriteLine("<========================")
                                                   End Sub))

            hostTransformMap(sh).Add(New Transform(Epilog,
                                                   Sub(host)
                                                       Console.WriteLine(host.WriteEndpoints())
                                                       Console.WriteLine("========================/>")
                                                   End Sub))
            hostTransformMap(sh).Add(DebugTransform)
            Return sh
        End Function

        Public Function RegisterService(ByVal serviceType As Type) As ServiceHostBase
            Return RegisterService(serviceType, Nothing)
        End Function

        Public Sub RegisterHost(ByVal host As ServiceHostBase)
            hostTransformMap(host) = New List(Of Transform)()

            'Register default transforms...
            hostTransformMap(host).Add(New Transform(Epilog,
                                                     Sub(h)
                                                         Console.WriteLine(host.WriteEndpoints())
                                                         Console.WriteLine("========================/>")
                                                     End Sub))
            hostTransformMap(host).Add(DebugTransform)
        End Sub

        Public Sub Open()
            For Each host In hostTransformMap.Keys
                Dim epilog = hostTransformMap(host).Where(
                    Function(t) t.Name = (Utilities.ServiceHostManager.Epilog)).First()
                hostTransformMap(host).Remove(epilog)

                For Each Transform In hostTransformMap(host)
                    Transform.Apply(host)
                Next Transform
                host.Open()

                epilog.Apply(host)
            Next host
        End Sub

        Public Sub Close()
            For Each host In hostTransformMap.Keys
                host.Close()
            Next host
        End Sub

    End Class
End Namespace
