﻿Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Activities
Imports Extensions.Utilities
Imports Microsoft.IdentityModel.Claims
Imports Microsoft.IdentityModel.Configuration
Imports Microsoft.IdentityModel.Tokens

Namespace SampleService
	Friend Class Program
		Shared Sub Main(ByVal args() As String)
			Dim wf = New Workflow1()

			Dim wfsh = New WorkflowServiceHost(wf, New Uri("http://localhost:8080/Echo"))

            Dim configuration = New ServiceConfiguration With {
                .ServiceCertificate = New X509Certificate2("SampleService.pfx", "a"),
                .IssuerNameRegistry = New MyTestRegistry(),
                .CertificateValidationMode = System.ServiceModel.Security.X509CertificateValidationMode.None,
                .ClaimsAuthorizationManager = New SimpleAuthorizationManager()}
			configuration.AudienceRestriction.AllowedAudienceUris.Add(wfsh.BaseAddresses.First())
			FederatedServiceCredentials.ConfigureServiceHost(wfsh, configuration)
			wfsh.Open()

			Console.WriteLine(wfsh.WriteEndpoints())

			Console.ReadLine()
			wfsh.Close()
		End Sub



	End Class

	Public Class MyTestRegistry
		Inherits IssuerNameRegistry
		Public Overrides Overloads Function GetIssuerName(ByVal securityToken As IdentityModel.Tokens.SecurityToken) As String
			Return "TestIssuer"
		End Function
	End Class


	Friend Class SimpleAuthorizationManager
		Inherits ClaimsAuthorizationManager
		Public Overrides Function CheckAccess(ByVal context As AuthorizationContext) As Boolean
			PrintCallerIdentity(CType(context.Principal.Identity, IClaimsIdentity))
			Return True
		End Function

		Private Sub PrintCallerIdentity(ByVal identity As IClaimsIdentity)
			Console.ForegroundColor = ConsoleColor.Green
			Console.WriteLine("Caller's name: " & identity.Name)
			For Each claim As Claim In identity.Claims
				Console.WriteLine("ClaimType  : " & claim.ClaimType.Split("/"c).Last())
				Console.WriteLine("ClaimValue : " & claim.Value)
				Console.WriteLine()
			Next claim
			Console.ResetColor()
			Do While identity.Actor IsNot Nothing
				Console.ForegroundColor = ConsoleColor.Yellow
				Console.WriteLine("Acting Via: " & identity.Actor.Name)
				For Each claim As Claim In identity.Actor.Claims
					Console.WriteLine("ClaimType  : " & claim.ClaimType.Split("/"c).Last())
					Console.WriteLine("ClaimValue : " & claim.Value)
					Console.WriteLine()
				Next claim
				identity = identity.Actor
				Console.ResetColor()
			Loop

			Console.WriteLine()
		End Sub

	End Class
End Namespace
